Durability = LibStub("AceAddon-3.0"):NewAddon("Durability", "AceEvent-3.0", "AceBucket-3.0", "AceHook-3.0")
local self, Durability = Durability, Durability
local _G = _G
local I = LibStub("LibItemUpgradeInfo-1.0")

local SlotDurStrs = {}
local SlotCheckedStrs = {}
local items = {
		"Head",
		"Shoulder",
		"Chest",
		"Waist",
		"Legs",
		"Feet",
		"Wrist",
		"Hands",
		"MainHand",
		"SecondaryHand"
}
local checkedItems = {
		"Head",
		"Neck",
		"Shoulder",
		"Chest",
		"Waist",
		"Legs",
		"Feet",
		"Wrist",
		"Hands",
		"Back",
		"Finger0",
		"Finger1",
		"Trinket0",
		"Trinket1",
		"MainHand",
		"SecondaryHand"
}

function GetDurStrings(name)
	if(not SlotDurStrs[name]) then
		local slot = _G["Character" .. name .. "Slot"]
		SlotDurStrs[name] = slot:CreateFontString(nil, "OVERLAY")
		font, _, flags = GameFontNormal:GetFont()
		SlotDurStrs[name]:SetFont(font, 14, "OUTLINE")
		SlotDurStrs[name]:SetPoint("BOTTOMRIGHT", slot, "BOTTOMRIGHT", 3, 0)
	end
	return SlotDurStrs[name]
end

function GetCheckedStrings(name)
	if(not SlotCheckedStrs[name]) then
		local slot = _G["Character" .. name .. "Slot"]
		SlotCheckedStrs[name] = slot:CreateFontString(nil, "OVERLAY")
		font, _, flags = GameFontNormal:GetFont()
		SlotCheckedStrs[name]:SetFont(font, 14, "OUTLINE")
		SlotCheckedStrs[name]:SetPoint("TOPLEFT", slot, "TOPLEFT", 1, 0)
	end
	return SlotCheckedStrs[name]
end

function GetThresholdColour(percent)
	if percent < 0 then
		return 1, 0, 0
	elseif percent <= 0.5 then
		return 1, percent * 2, 0
	elseif percent >= 1 then
		return 0, 1, 0
	else
		return 2 - percent * 2, 1, 0
	end
end

function Durability:UpdateDurability()
	for _, item in ipairs(items) do
		local id, _ = GetInventorySlotInfo(item .. "Slot")
		local v1, v2 = GetInventoryItemDurability(id)
		v1, v2 = tonumber(v1) or 0, tonumber(v2) or 0
		local percent = v1 / v2
		local SlotDurStr = GetDurStrings(item)
		
		if ((v2 ~= 0) and (percent ~= 1)) then
				
			SlotDurStr:SetText('')
			if(ceil(percent * 100) < 100)then
				SlotDurStr:SetTextColor(GetThresholdColour(percent))
				SlotDurStr:SetText(ceil(percent * 100) .. "%")
			end
		else
			SlotDurStr:SetText('')
		end
	end
end

local function getArtifactIlvl()
	local mainIlvl, offIlvl = 0, 0
	local mainQ = GetInventoryItemQuality("player", 16)
	local offQ = GetInventoryItemQuality("player", 17)
	
	if (mainQ or offQ) and (mainQ == 6 or offQ == 6) then
		mainIlvl = I:GetUpgradedItemLevel(GetInventoryItemLink("player", 16)) or 0
		offIlvl = I:GetUpgradedItemLevel(GetInventoryItemLink("player", 17)) or 0
	end
	return math.max(mainIlvl, offIlvl)
end

function Durability:CheckItem()
	for _, item in ipairs(checkedItems) do
		local id, _ = GetInventorySlotInfo(item .. "Slot")
		local itemLink = GetInventoryItemLink("player", id)
		
		if itemLink then  --如果该部位有装备物品
			local str = ""
			-- ilevel
			local SlotCheckedStrs = GetCheckedStrings(item)
			local iLevel = I:GetUpgradedItemLevel(itemLink)
			local iQualityColor = select(4, GetItemQualityColor(GetInventoryItemQuality("player", id)))
			
			if (id == 16 or id == 17) and getArtifactIlvl() ~= 0 then -- Artifact equipped
				iLevel = getArtifactIlvl()
			end
			
			str = "|c" .. iQualityColor .. iLevel .. "|r"
			
			-- gem & enchant
			if iLevel >= 600 then --不判断老装备是否附魔镶嵌宝石
				local itemStats = GetItemStats(itemLink)
				local g = 1
				local e = 1
			
				for k,v in pairs(itemStats) do
					if k == "EMPTY_SOCKET_PRISMATIC" then
						if not(GetItemGem(itemLink, 1)) then g = 0 end
					end
				end
				table.wipe(itemStats)
			
				if item == "Neck" or item == "Back" or item == "Finger0" or item == "Finger1" then
					local data = select(3, strsplit("|", itemLink))
					local enchant = select(3, strsplit(':', data))
					if enchant == "" then e = 0 end 
				end
			
				local result = tonumber(g .. e)
				if result == 10 then
					str = str .. " |cffff0000E|r"
				elseif result == 01 then
					str = str .. " |cffff0000G|r"
				elseif result == 00 then
					str = str .. " |cffff0000EG|r"
				end
			end
			
			SlotCheckedStrs:SetText(str)
		else
			GetCheckedStrings(item):SetText('')
		end
	end
end

function Durability:OnEnable()
	self:SecureHookScript(CharacterFrame, "OnShow", "CharacterFrame_OnShow")
	self:SecureHookScript(CharacterFrame, "OnHide", "CharacterFrame_OnHide")
end

function Durability:CharacterFrame_OnShow()
	self:RegisterEvent("UNIT_INVENTORY_CHANGED", "UpdateDurability")
	self:RegisterBucketEvent("UPDATE_INVENTORY_DURABILITY", 0.5, "UpdateDurability")
	self:UpdateDurability()
	
	self:RegisterEvent("UNIT_INVENTORY_CHANGED", "CheckItem")
	self:CheckItem()
end

function Durability:CharacterFrame_OnHide()
	self:UnregisterEvent("UNIT_INVENTORY_CHANGED")
	self:UnregisterBucket("UPDATE_INVENTORY_DURABILITY")
end